<?php
// Functionality for bridging WordPress to vBulletin.
// Tailored for WordPress 2.0.1, but may work on more recent once as well.
// Written by Robert Andersson <robert@profundis.se>
// http://www.profundis.se/stuff/wordpress_vbulletin_bridge/
//
// INSTALLATION
// ============
// * We must add a new column in the post table on the WP side to hold which
//   thread that corresponds to the post:
//   ALTER TABLE `wp_posts` ADD `vb_threadid` INT(10); 
// * FILE: wp-admin/admin-functions.php
//   * First few lines: 
//     require_once(ABSPATH."wordpress_vbulletin_bridge.php");
//   * Before returning in the functions write_post() and edit_post():
//     wp2vb_newpost($post_ID);
// * FILE: wp-includes/comment-functions.php
//   * First few lines:
//     require_once(ABSPATH."wordpress_vbulletin_bridge.php");
//   * In function get_comments_number(), ca line 230, replace line:
//     $comment_count_cache[$id] = $wpdb->get_var(...);
//     ... with ...
//     $comment_count_cache[$id] = wp2vb_get_comments_number($post_id);
//   * In function comments_popup_link(), ca line 280, replace line:
//     $comment_count_cache[$id] = $wpdb->get_var(...);
//     ... with ...
//     $comment_count_cache[$id] = wp2vb_get_comments_number($id);
//     ... and ...
//     echo get_permalink() . '#respond'; (ca line 310)
//     ... with ...
//     echo wp2vb_get_comments_link_respond($id);
//   * In function get_comments_link(), ca line 240, replace return with:
//     return wp2vb_get_comments_link($GLOBALS['id']);
// * FILE: wp-includes/functions.php
//     * Comment out this line in function update_post_caches(), ca line 1410:
//       $comment_count_cache[$posts[$i]->ID] = $posts[$i]->comment_count;
// * FILE: wp-content/themes/*/comments.php
//     * This will depend on your template, but insert a PHP block somewhere
//       before anything is written, and modify as appropriate:
//       <?php
//       global $wp2vb_activated,$wp2vb_vb_forum_name;
//       if($wp2vb_activated) {
//         echo "<h2 id=\"comments\">";
//         comments_number(__('No Comments'), __('1 Comment'), __('% Comments'));
//         echo "</h2>\n";
//         $uri = wp2vb_get_comments_link($id);
//         echo "<p>Leave and read comments on the <a href=\"$uri\">$wp2vb_vb_forum_name</a>.</p>\n";
//         return;
//       }
//       ?\>
//
// BUGS / ISSUES
// =============
// * WordPress uses UTF-8 and vBulletin ISO-8859-1 encoding
// * Don't work well with javascript popup comments, whatever that is
// * RSS, TrackBall etc, is magic to me, so...
// * Many of the advanced features of WP may be incompatiable with the bridge
//
// STATUS
// ======
// wp2vb_newpost() - done: generalize format of post content?
// wp2vb_get_comments_number - done
// wp2vb_get_comments_link - done
// wp2vb_get_comments_link_respond - done
// wp2vb_comments_template() - dummy: please adapt this to your needs
// wp2vb_wp_getpost() - done
// wp2vb_wp_getcomments() - done
// wp2vb_wp_ispublished() - done
// wp2vb_wp_post_uri() - done
// wp2vb_wp_get_vb_threadid() - done
// wp2vb_wp_save_vb_threadid() - done
// wp2vb_vb_newthread() - done
// wp2vb_vb_newreply() - done
// wp2vb_vb_commentcount() - done
// wp2vb_vb_thread_uri() - done: should we have goto=newpost?
// wp2vb_vb_thread_uri_respond() - dummy: where should this go to?
// wp2vb_vb_dblink() - done
// wp2vb_extract_summary() - done: what should max_chars be?
// wp2vb_array2set() - done
// wp2vb_html2vbcode() - done


// CONFIGURATION
require_once("wordpress_vbulletin_bridge-config.php");


// Main hook function - call when a WorldPress post is saved/published
function wp2vb_newpost($wp_post_id)
{
  global $wp2vb_activated, $wp2vb_bloglink_name;

  if(!$wp2vb_activated) {
    return;
  }

  // Fetch the WorldPress post
  $wp_post = wp2vb_wp_getpost($wp_post_id);

  // Have we already added a thread to this post? If so, we're done
  if($wp_post['vb_threadid']) {
    return;
  }

  // Don't add a thread if the post haven't been published yet
  if(!wp2vb_wp_ispublished($wp_post)) {
    return;
  }

  // Ok, create the thread then
  $link = "[url=".wp2vb_wp_post_uri($wp_post_id)."]${wp2vb_bloglink_name}[/url]";
  $subject = $wp_post['post_title'];
  if(!($summary = $wp_post['post_excerpt'])) {
    // The entry does not have an excerpt so we try to construct one
    $summary = wp2vb_extract_summary($wp_post['post_content']);
  }
  $body = $summary."\n\n".$link;
  $time = strtotime($wp_post['post_date']);
  $vb_thread_id = wp2vb_vb_newthread($subject, $body, $time);
  wp2vb_wp_save_vb_threadid($wp_post_id, $vb_thread_id);
 
  // If there are any comments, we will import them as well
  $wp_comment_list = wp2vb_wp_getcomments($wp_post_id);
  if($wp_comment_list) {
    foreach($wp_comment_list as $comment) {
      $body = wp2vb_html2vbcode($comment['comment_content']);
      $name = $comment['comment_author'];
      $email = $comment['comment_author_email'];
      $time = strtotime($comment['comment_date']);
      $ip = $comment['comment_author_IP'];
      wp2vb_vb_newreply($vb_thread_id, $body, $name, $email, $time, $ip);
    }
  }
}


function wp2vb_get_comments_number($wp_post_id)
{
  global $wp2vb_activated;

  if(!$wp2vb_activated) {
    // original WP code
    global $wpdb, $comment_count_cache;
    if (!isset($comment_count_cache[$comment_id])) {
      $comment_count_cache[$wp_post_id] = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->comments WHERE comment_post_ID=$wp_post_id AND comment_approved = '1'");
    }
    return $comment_count_cache[$wp_post_id];
  }

  if($vb_thread_id = wp2vb_wp_get_vb_threadid($wp_post_id)) {
    return wp2vb_vb_commentcount($vb_thread_id);
  }
  else {
    return "(error-no_vb_threadid_for_this_post)";
  }
}


function wp2vb_get_comments_link($wp_post_id)
{
  global $wp2vb_activated;

  if(!$wp2vb_activated) {
    return get_permalink() . '#comments'; // original WP code
  }

  if($vb_thread_id = wp2vb_wp_get_vb_threadid($wp_post_id)) {
    return wp2vb_vb_thread_uri($vb_thread_id);
  }
  else {
    return "#error-no_vb_threadid_for_this_post";
  }
}


function wp2vb_get_comments_link_respond($wp_post_id)
{
  global $wp2vb_activated;

  if(!$wp2vb_activated) {
    return get_permalink() . '#respond'; // original WP code
  }

  if($vb_thread_id = wp2vb_wp_get_vb_threadid($wp_post_id)) {
    return wp2vb_vb_thread_uri_respond($vb_thread_id);
  }
  else {
    return "#error-no_vb_threadid_for_this_post";
  }
}


// WordPress Functions - most specifics is isolated here
// =====================================================

function wp2vb_wp_getpost($post_id)
{
  global $wpdb;

  $t = $wpdb->posts;
  $query = "SELECT * FROM $t WHERE `ID`=$post_id";
  $post = $wpdb->get_row($query, ARRAY_A);

  return $post;
}

// comment_author, comment_author_email, comment_date, commnet_content
function wp2vb_wp_getcomments($post_id)
{
  global $wpdb;

  $t = $wpdb->comments;
  $query = "SELECT * FROM $t WHERE `comment_post_ID`=$post_id";
  $comment_list = $wpdb->get_results($query, ARRAY_A);

  return $comment_list;
}

function wp2vb_wp_ispublished($post)
{
  return $post['post_status'] == 'publish';
}

function wp2vb_wp_post_uri($wp_post_id)
{
  return post_permalink($wp_post_id);
}

function wp2vb_wp_get_vb_threadid($wp_post_id)
{
  global $wpdb;
  static $vb_threadid_cache = array();

  if($vb_threadid_cache[$wp_post_id] === null) {
    $t = $wpdb->posts;
    $query = "SELECT `vb_threadid` FROM $t WHERE `ID`=$wp_post_id";
    $vb_threadid_cache[$wp_post_id] = $wpdb->get_var($query);
  }
  return $vb_threadid_cache[$wp_post_id];
}

function wp2vb_wp_save_vb_threadid($wp_post_id, $vb_thread_id)
{
  global $wpdb;

  $t = $wpdb->posts;
  $query = "UPDATE $t SET `vb_threadid`=$vb_thread_id WHERE `ID`=$wp_post_id";
  $wpdb->query($query);
}


// vBulletin Functions - most specifics is isolated here
// =====================================================

function wp2vb_vb_newthread($subject, $body, $time)
{
  global $wp2vb_vb_forum_id,$wp2vb_vb_user_id,$wp2vb_vb_user_name;

  $dblink = wp2vb_vb_dblink();
  $ip = $_SERVER['REMOTE_ADDR'] ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0';

  // Make the new thread (it will not contain the initial post)
  $values = array('title' => $subject,
                  'lastpost' => $time,
                  'forumid' => $wp2vb_vb_forum_id,
                  'open' => 1,
                  'postuserid' => $wp2vb_vb_user_id,
                  'postusername' => $wp2vb_vb_user_name,
                  'lastposter' => $wp2vb_vb_user_name,
                  'dateline' => $time,
                  'visible' => 1,
                  );
  $query = "INSERT INTO thread SET ".wp2vb_array2set($values);
  mysql_query($query, $dblink);
  $thread_id = mysql_insert_id($dblink);

  // Make the first post and set the id in the thread
  // XXX: should 'showsignature' be on?
  $values = array('threadid' => $thread_id,
		  'username' => $wp2vb_vb_user_name,
		  'userid' => $wp2vb_vb_user_id,
		  'title' => $subject,
		  'dateline' => $time,
		  'pagetext' => $body,
		  'visible' => 1,
		  'ipaddress' => $ip,
		  'allowsmilie' => 1,
		  'iconid' => 1,
		  );
  $query = "INSERT INTO post SET ".wp2vb_array2set($values);
  mysql_query($query, $dblink);
  $post_id = mysql_insert_id($dblink);
  $query = "UPDATE thread SET firstpostid=$post_id WHERE threadid=$thread_id";
  mysql_query($query, $dblink); 

  // Update forum's thread count and stuff
  $values = array('lastpost' => $time,
		  'lastposter' => $wp2vb_vb_user_name,
		  'lastthread' => $subject,
		  'lastthreadid' => $thread_id,
		  'lasticonid' => 1,
		  );
  $set_clause = "`threadcount`=`threadcount`+1,".wp2vb_array2set($values);
  $query = "UPDATE forum SET $set_clause WHERE forumid=$wp2vb_vb_forum_id";
  mysql_query($query, $dblink);

  return $thread_id;
}

function wp2vb_vb_newreply($thread_id, $body, $author_name, $author_email, $time, $ip)
{
  global $wp2vb_vb_forum_id,$wp2vb_vb_user_id,$wp2vb_vb_user_name;

  $dblink = wp2vb_vb_dblink();

  // We will need some stuff from the thread :(
  $query = "SELECT firstpostid,title FROM thread WHERE threadid=$thread_id";
  $result = mysql_query($query, $dblink);
  $thread = mysql_fetch_assoc($result);

  // See if we can find a user id
  if($author_email) {
    $email_esc = mysql_real_escape_string($author_email, $dblink);
    $query = "SELECT userid,username FROM user WHERE email='$email_esc'";
    $result = mysql_query($query, $dblink);
    $user = mysql_fetch_assoc($result);
    if($user) {
      $author_vb_userid = $user['userid'];
      $author_name = $user['username'];
    }
  }
  else {
    $author_vb_userid = 0;
  }

  // Create the post
  $values = array('threadid' => $thread_id,
		  'parentid' => $thread['firstpostid'],
                  'username' => $author_name,
                  'userid' => $author_vb_userid,
                  //'title' => $subject,
                  'dateline' => $time,
                  'pagetext' => $body,
                  'visible' => 1,
                  'ipaddress' => $ip,
                  'allowsmilie' => 1,
                  'iconid' => 1,
                  );
  $query = "INSERT INTO post SET ".wp2vb_array2set($values);
  mysql_query($query, $dblink);
  $post_id = mysql_insert_id($dblink);

  // Update the thread
  $values = array('lastpost' => $time,
		  'lastposter' => $author_name,
		  );
  $set_clause = "`replycount`=`replycount`+1,".wp2vb_array2set($values);
  $query = "UPDATE thread SET $set_clause WHERE threadid=$thread_id";
  mysql_query($query, $dblink);

  // Update forum's thread count and stuff
  $values = array('lastpost' => $time,
		  'lastposter' => $author_name,
                  'lastthread' => $thread['title'],
                  'lastthreadid' => $thread_id,
                  'lasticonid' => 1,
                  );
  $set_clause = wp2vb_array2set($values);
  $query = "UPDATE forum SET $set_clause WHERE forumid=$wp2vb_vb_forum_id";
  mysql_query($query, $dblink);

  return $post_id;
}

function wp2vb_vb_commentcount($thread_id)
{
  $dblink = wp2vb_vb_dblink();

  $query = "SELECT replycount FROM thread WHERE threadid=$thread_id";
  $result = mysql_query($query, $dblink);
  $thread = mysql_fetch_assoc($result);
  return $thread['replycount'];
}

function wp2vb_vb_thread_uri($thread_id)
{
  global $wp2vb_vb_uri_prefix;
  return $wp2vb_vb_uri_prefix."showthread.php?t=$thread_id";
}

function wp2vb_vb_thread_uri_respond($thread_id)
{
  global $wp2vb_vb_uri_prefix;
  return $wp2vb_vb_uri_prefix."showthread.php?t=$thread_id";
}

function wp2vb_vb_dblink()
{
  global $wp2vb_db_host, $wp2vb_db_name, $wp2vb_db_user, $wp2vb_db_password;
  static $link = null;

  if($link == null) {
    $link = mysql_connect($wp2vb_db_host,$wp2vb_db_user,$wp2vb_db_password);
    if($link) {
      mysql_select_db($wp2vb_db_name, $link);
    }
  }

  return $link;
}


// Misc functions
// ==============

function wp2vb_extract_summary($text, $max_chars = 150, $max_paragraphs = 1)
{
  // Remove any markup
  $text = trim(strip_tags($text));

  // Extract and limit paragraphs
  $paragraphs = preg_split("/(\r?\n){2,}/m", $text, -1, PREG_SPLIT_NO_EMPTY);
  $paragraphs = array_slice($paragraphs, 0, $max_paragraphs);

  // See how many of the paragraphs we can use
  if($max_chars > 0) {
    do {
      $last_paragraph = array_pop($paragraphs);
    } while(strlen(join("", $paragraphs)) > $max_chars);
    // If the last paragraph is too long, we need to truncate it
    $max_last_p = $max_chars - strlen(join("", $paragraphs)); // ought to be positive
    if(strlen($last_paragraph) > $max_last_p) {
      // Split into sentences
      $sentences = preg_split("/(?<=.[\.!?:]\s)/", trim($last_paragraph), -1, PREG_SPLIT_NO_EMPTY);
      do {
	$last_sentence = array_pop($sentences);
      } while(strlen(join("", $sentences)) > $max_last_p);
      // Put back the last sentence if there is still some room
      if(strlen(join("", $sentences)) < $max_last_p) {
	array_push($sentences, $last_sentence);
      }
      // Reconstruct last paragraph
      $last_paragraph = trim(join("", $sentences))." ...";
    }
    // Put back the last paragraph if there is still some room
    if(strlen(join("", $paragraphs)) < $max_chars) {
      array_push($paragraphs, $last_paragraph);
    }
  }

  // Reconstruct the text
  $text = join("\r\n\r\n", $paragraphs);

  return $text;
}


function wp2vb_array2set($column_values)
{
  // Our buffer array
  $set_list = array();

  // For each kay-value pair
  foreach($column_values as $key => $value)
    $set_list[] = "`$key`=".(is_null($value) ? 'NULL' : "'".mysql_escape_string($value)."'");

  // Return a joined string
  return join(',', $set_list);
}

function wp2vb_html2vbcode($text)
{
  // a, abbr, code, blackquote, acronym, b, i, em, strong, strike
  $text = preg_replace("/<a\s.*?href=[\"\']?(.+?)(>|[\"\'\s].*?\>)(.*?)<\/a>/i", "[url=\"$1\"]$3[/url]", $text);
  $text = preg_replace("/<(\/)?code.*?>/i", "[$1code]", $text);
  $text = preg_replace("/<blockquote\s.*?cite=[\"\']?(.+?)(>|[\"\'\s].*?\>)(.*?)<\/blockquote>/i", "[quote=\"$1\"]$3[/quote]", $text);
  $text = preg_replace("/<(\/)?b.*?>/i", "[$1b]", $text);
  $text = preg_replace("/<(\/)?i.*?>/i", "[$1i]", $text);
  $text = preg_replace("/<(\/)?(em|strong).*?>/i", "[$1highlight]", $text);
  $text = preg_replace("/<\/?(abbr|acronym|strike).*?>/i", "", $text);
  return $text;
}


?>